using Microsoft.Extensions.Logging;
using ShipyardBusinessLogic.MailWorker;
using ShipyardContracts.BindingModels;
using ShipyardContracts.BusinessLogicsContracts;
using ShipyardContracts.SearchModels;
using ShipyardContracts.StoragesContracts;
using ShipyardContracts.ViewModels;
using ShipyardDataModels.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ShipyardBusinessLogic.BusinessLogics
{
    public class OrderLogic:IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly AbstractMailWorker _mailWorker;
        private readonly IClientLogic _clientLogic;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, AbstractMailWorker mailWorker,IClientLogic clientLogic)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _mailWorker = mailWorker;
            _clientLogic = clientLogic;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}",model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public bool CreateOrder(OrderBindingModel model) 
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Insert operation failed. Order status incorrect.");
                return false;
            }
            model.Status = OrderStatus.Принят;
            var result = _orderStorage.Insert(model);
            if (result == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            SendOrderMessage(result.ClientId, $"Верфь, Заказ №{result.Id}", $"Заказ №{result.Id} от {result.DateCreate} на сумму {result.Sum:0.00} принят");
            return true;
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.ShipId < 0) 
            {
                throw new ArgumentNullException("Некорректный идентификатор у корабля", nameof(model.ShipId));
            }
            if (model.Count <= 0) 
            {
                throw new ArgumentNullException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0) 
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}. Sum:{ Sum}. ShipId: { ShipId}", model.Id, model.Sum, model.ShipId);
        }
        public bool StatusUpdate(OrderBindingModel model, OrderStatus newStatus)
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (viewModel.Status + 1 != newStatus)
            {
                _logger.LogWarning("Status update to " + newStatus.ToString() + " operation failed. Order status incorrect.");
                return false;
            }
            model.Status = newStatus;
            if (viewModel.ImplementerId.HasValue)
            {
                model.ImplementerId = viewModel.ImplementerId;
            }
            if (model.Status == OrderStatus.Готов) model.DateImplement = DateTime.Now;
            else 
            {
                model.DateImplement = viewModel.DateImplement;
            }
            CheckModel(model,false);
            var result = _orderStorage.Update(model);
            if (result == null)
            {
                model.Status--;
                _logger.LogWarning("Update operation failed");
                return false;
            }
            SendOrderMessage(result.ClientId, $"Верфь, Заказ №{result.Id}", $"Заказ №{model.Id} изменен статус на {result.Status}");
            return true;
        }


        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        private bool SendOrderMessage(int clientId, string subject, string text)
        {
            var client = _clientLogic.ReadElement(new() { Id = clientId });
            if (client == null)
            {
                return false;
            }
            _mailWorker.MailSendAsync(new()
            {
                MailAddress = client.Email,
                Subject = subject,
                Text = text
            });
            return true;
        }
    }
}
